// Mesure de tension avec l'ADC101 et une tension de référence de 3,3V (REF3231).
// Puis calcul et affichage de la vitesse du vent en km/h et en m/sec
// La calcul se fait à partir de 5 échantillon successifs.
// Les calculs se font en float, puis avant l'affiche passage en int
// Fichier Vitesse_vent_03.cpp
#include <iostream> 
#include <bcm2835.h>
using namespace std;

#define clk_div BCM2835_I2C_CLOCK_DIVIDER_2500

// g++ Vitesse_vent_03.cpp -l bcm2835 -o Vitesse_vent_03 

void init_I2c_bcm2835()
{
	if (!bcm2835_init())
    		{
      		printf("bcm2835_init failed. Are you running as root??\n");
    		}
      	if (!bcm2835_i2c_begin())
      		{
        	printf("bcm2835_i2c_begin failed. Are you running as root??\n");
      		}
	bcm2835_i2c_setClockDivider(clk_div);			// Division de l'horloge Rpi pour obtenir une vitesse de 100KHz
}

int main(void)
{
	double mesure;
	unsigned int slave_address=0x54; // Adresse du DS1621
	char i2cOut[3]; 	// Tableau des données I2C à sortir 
	char i2cIn[2];		// Tableau des données I2C entrées
	int resultat_ADC =0;
	float tension=0;
	float vitesseKMH=0;
	float vitesseMS=0;
	int vitesse_kmh=0;
	int vitesse_ms=0;


	init_I2c_bcm2835();
	bcm2835_i2c_setSlaveAddress(slave_address);	// Affectation de l'adresse de l'esclave 

	i2cOut[0] = 0x00;
	bcm2835_i2c_write(i2cOut, 1); 	// Prépositionne la valeur du pointeur 0x00 Pour être en mode "Conversion Result)
	delay(1000);					// 1 seconde avant première lecture

	while(1)
	{
	resultat_ADC =0;
	tension = 0;
	for (int i=0; i<5; i++)
		{		 
		bcm2835_i2c_read(i2cIn,2);
		resultat_ADC = ((i2cIn[1]>>2)& 0b00111111) + (64* (i2cIn[0] & 0b00001111));
		tension += resultat_ADC*3.3/1023;
		delay(10);	
		}
	tension/=5.0;
	vitesseKMH = (tension*23.58) -29.24;
	if (vitesseKMH<0) vitesseKMH=0.0;
	vitesseMS = (vitesseKMH*1000)/3600;
	if (vitesseMS<0) vitesseMS=0.0;
	vitesse_kmh = (int) vitesseKMH;
	vitesse_ms = (int) vitesseMS;
	cout<<"Vitesse du vent =  "<<vitesse_kmh<<" km/h =  "<<vitesse_ms<<" m/sec "<<endl;
	delay(1000);									// 1 seconde entre chaque conversion
	}

	bcm2835_close();
	return 0;
}
